package sun.awt.windows;

import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.HashMap;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Color;
import java.awt.Insets;
import java.util.Iterator;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.Map;

public final class ThemeReader
{
    private static final Map<String, Long> widgetToTheme;
    private static final ReadWriteLock readWriteLock;
    private static final Lock readLock;
    private static final Lock writeLock;
    private static volatile boolean valid;
    private static volatile boolean isThemed;
    static volatile boolean xpStyleEnabled;
    
    public ThemeReader() {
    }
    
    static void flush() {
        ThemeReader.valid = false;
    }
    
    private static native boolean initThemes();
    
    public static boolean isThemed() {
        ThemeReader.writeLock.lock();
        try {
            ThemeReader.isThemed = initThemes();
            return ThemeReader.isThemed;
        }
        finally {
            ThemeReader.writeLock.unlock();
        }
    }
    
    public static boolean isXPStyleEnabled() {
        return ThemeReader.xpStyleEnabled;
    }
    
    private static Long getThemeImpl(String s) {
        Long n = (Long)ThemeReader.widgetToTheme.get(s);
        if (n == null) {
            int index = s.indexOf("::");
            if (index > 0) {
                setWindowTheme(s.substring(0, index));
                n = Long.valueOf(openTheme(s.substring(index + 2)));
                setWindowTheme(null);
            }
            else {
                n = Long.valueOf(openTheme(s));
            }
            ThemeReader.widgetToTheme.put(s, n);
            return n;
        }
        else {
            return n;
        }
    }
    
    private static Long getTheme(String s) {
        if (!ThemeReader.isThemed) {
            throw new IllegalStateException("Themes are not loaded");
        }
        if (!ThemeReader.valid) {
            ThemeReader.readLock.unlock();
            ThemeReader.writeLock.lock();
            try {
                if (!ThemeReader.valid) {
                    Iterator<Long> iterator = ThemeReader.widgetToTheme.values().iterator();
                    while (iterator.hasNext()) {
                        closeTheme((long)iterator.next());
                    }
                    ThemeReader.widgetToTheme.clear();
                    ThemeReader.valid = true;
                }
            }
            finally {
                ThemeReader.readLock.lock();
                ThemeReader.writeLock.unlock();
            }
        }
        Long themeImpl = (Long)ThemeReader.widgetToTheme.get(s);
        if (themeImpl == null) {
            ThemeReader.readLock.unlock();
            ThemeReader.writeLock.lock();
            try {
                themeImpl = getThemeImpl(s);
            }
            finally {
                ThemeReader.readLock.lock();
                ThemeReader.writeLock.unlock();
            }
        }
        return themeImpl;
    }
    
    private static native void paintBackground(int[] p0, long p1, int p2, int p3, int p4, int p5, int p6, int p7, int p8);
    
    public static void paintBackground(int[] array, String s, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
        ThemeReader.readLock.lock();
        try {
            paintBackground(array, (long)getTheme(s), n, n2, n3, n4, n5, n6, n7);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native Insets getThemeMargins(long p0, int p1, int p2, int p3);
    
    public static Insets getThemeMargins(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getThemeMargins((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native boolean isThemePartDefined(long p0, int p1, int p2);
    
    public static boolean isThemePartDefined(String s, int n, int n2) {
        ThemeReader.readLock.lock();
        try {
            return isThemePartDefined((long)getTheme(s), n, n2);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native Color getColor(long p0, int p1, int p2, int p3);
    
    public static Color getColor(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getColor((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native int getInt(long p0, int p1, int p2, int p3);
    
    public static int getInt(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getInt((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native int getEnum(long p0, int p1, int p2, int p3);
    
    public static int getEnum(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getEnum((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native boolean getBoolean(long p0, int p1, int p2, int p3);
    
    public static boolean getBoolean(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getBoolean((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native boolean getSysBoolean(long p0, int p1);
    
    public static boolean getSysBoolean(String s, int n) {
        ThemeReader.readLock.lock();
        try {
            return getSysBoolean((long)getTheme(s), n);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native Point getPoint(long p0, int p1, int p2, int p3);
    
    public static Point getPoint(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getPoint((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native Dimension getPosition(long p0, int p1, int p2, int p3);
    
    public static Dimension getPosition(String s, int n, int n2, int n3) {
        ThemeReader.readLock.lock();
        try {
            return getPosition((long)getTheme(s), n, n2, n3);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native Dimension getPartSize(long p0, int p1, int p2);
    
    public static Dimension getPartSize(String s, int n, int n2) {
        ThemeReader.readLock.lock();
        try {
            return getPartSize((long)getTheme(s), n, n2);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    private static native long openTheme(String p0);
    
    private static native void closeTheme(long p0);
    
    private static native void setWindowTheme(String p0);
    
    private static native long getThemeTransitionDuration(long p0, int p1, int p2, int p3, int p4);
    
    public static long getThemeTransitionDuration(String s, int n, int n2, int n3, int n4) {
        ThemeReader.readLock.lock();
        try {
            return getThemeTransitionDuration((long)getTheme(s), n, n2, n3, n4);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    public static native boolean isGetThemeTransitionDurationDefined();
    
    private static native Insets getThemeBackgroundContentMargins(long p0, int p1, int p2, int p3, int p4);
    
    public static Insets getThemeBackgroundContentMargins(String s, int n, int n2, int n3, int n4) {
        ThemeReader.readLock.lock();
        try {
            return getThemeBackgroundContentMargins((long)getTheme(s), n, n2, n3, n4);
        }
        finally {
            ThemeReader.readLock.unlock();
        }
    }
    
    static {
        widgetToTheme = new HashMap();
        readWriteLock = new ReentrantReadWriteLock();
        readLock = ThemeReader.readWriteLock.readLock();
        writeLock = ThemeReader.readWriteLock.writeLock();
        ThemeReader.valid = false;
    }
}
